@@ -1,13 +1,34 @@
 /*
- * GdkPixbufOverride.c
+ * java-gnome, a UI library for writing GTK and GNOME programs from Java!
  *
- * Copyright (c) 2008-2009 Operational Dynamics Consulting Pty Ltd, and Others
+ * Copyright © 2008-2010 Operational Dynamics Consulting, Pty Ltd and Others
  *
- * The code in this file, and the library it is a part of, are made available
- * to you by the authors under the terms of the "GNU General Public Licence,
- * version 2" plus the "Classpath Exception" (you may link to this code as a
- * library into other programs provided you don't make a derivation of it).
- * See the LICENCE file for the terms governing usage and redistribution.
+ * The code in this file, and the program it is a part of, is made available
+ * to you by its authors as open source software: you can redistribute it
+ * and/or modify it under the terms of the GNU General Public License version
+ * 2 ("GPL") as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE. See the GPL for more details.
+ *
+ * You should have received a copy of the GPL along with this program. If not,
+ * see http://www.gnu.org/licenses/. The authors of this program may be
+ * contacted through http://java-gnome.sourceforge.net/.
+ *
+ * Linking this library statically or dynamically with other modules is making
+ * a combined work based on this library. Thus, the terms and conditions of
+ * the GPL cover the whole combination. As a special exception (the
+ * "Claspath Exception"), the copyright holders of this library give you
+ * permission to link this library with independent modules to produce an
+ * executable, regardless of the license terms of these independent modules,
+ * and to copy and distribute the resulting executable under terms of your
+ * choice, provided that you also meet, for each linked independent module,
+ * the terms and conditions of the license of that module. An independent
+ * module is a module which is not derived from or based on this library. If
+ * you modify this library, you may extend the Classpath Exception to your
+ * version of the library, but you are not obligated to do so. If you do not
+ * wish to do so, delete this exception statement from your version.
  */
 
 #include <jni.h>
@@ -159,3 +180,66 @@ Java_org_gnome_gdk_GdkPixbufOverride_gdk_1pixbuf_1new_1from_1stream
 
 	return (jlong) result;
 }
+
+JNIEXPORT jint JNICALL
+Java_org_gnome_gdk_GdkPixbufOverride_gdk_1pixbuf_1get_1file_1info_1X
+(
+	JNIEnv* env,
+	jclass cls,
+	jstring _filename
+)
+{
+	const GdkPixbufFormat* format;
+	const gchar* filename;
+	int x;
+
+	// convert parameter filename
+	filename = (const gchar*) bindings_java_getString(env, _filename);
+	if (filename == NULL) {
+		return 0; // Java Exception already thrown
+	}
+
+	// call function
+	format = gdk_pixbuf_get_file_info(filename, &x, NULL);
+	if (format == NULL) {
+		x = -1;
+	}
+
+	// cleanup parameter filename
+	bindings_java_releaseString(filename);
+
+	// translate return value to JNI type
+	return (jint) x;
+}
+
+JNIEXPORT jint JNICALL
+Java_org_gnome_gdk_GdkPixbufOverride_gdk_1pixbuf_1get_1file_1info_1Y
+(
+	JNIEnv* env,
+	jclass cls,
+	jstring _filename
+)
+{
+	const GdkPixbufFormat* format;
+	const gchar* filename;
+	int y;
+
+	// convert parameter filename
+	filename = (const gchar*) bindings_java_getString(env, _filename);
+	if (filename == NULL) {
+		return 0; // Java Exception already thrown
+	}
+
+	// call function
+	format = gdk_pixbuf_get_file_info(filename, NULL, &y);
+	if (format == NULL) {
+		y = -1;
+	}
+
+	// cleanup parameter filename
+	bindings_java_releaseString(filename);
+
+	// translate return value to JNI type
+	return (jint) y;
+}
+
