@@ -1,13 +1,34 @@
 /*
- * GValue.c
+ * java-gnome, a UI library for writing GTK and GNOME programs from Java!
  *
- * Copyright (c) 2006-2009 Operational Dynamics Consulting Pty Ltd
- * 
- * The code in this file, and the library it is a part of, are made available
- * to you by the authors under the terms of the "GNU General Public Licence,
- * version 2" plus the "Classpath Exception" (you may link to this code as a
- * library into other programs provided you don't make a derivation of it).
- * See the LICENCE file for the terms governing usage and redistribution.
+ * Copyright © 2006-2010 Operational Dynamics Consulting, Pty Ltd
+ *
+ * The code in this file, and the program it is a part of, is made available
+ * to you by its authors as open source software: you can redistribute it
+ * and/or modify it under the terms of the GNU General Public License version
+ * 2 ("GPL") as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE. See the GPL for more details.
+ *
+ * You should have received a copy of the GPL along with this program. If not,
+ * see http://www.gnu.org/licenses/. The authors of this program may be
+ * contacted through http://java-gnome.sourceforge.net/.
+ *
+ * Linking this library statically or dynamically with other modules is making
+ * a combined work based on this library. Thus, the terms and conditions of
+ * the GPL cover the whole combination. As a special exception (the
+ * "Claspath Exception"), the copyright holders of this library give you
+ * permission to link this library with independent modules to produce an
+ * executable, regardless of the license terms of these independent modules,
+ * and to copy and distribute the resulting executable under terms of your
+ * choice, provided that you also meet, for each linked independent module,
+ * the terms and conditions of the license of that module. An independent
+ * module is a module which is not derived from or based on this library. If
+ * you modify this library, you may extend the Classpath Exception to your
+ * version of the library, but you are not obligated to do so. If you do not
+ * wish to do so, delete this exception statement from your version.
  */
 
 #include <glib.h>
@@ -47,7 +68,7 @@ Java_org_gnome_glib_GValue_g_1type_1name
 
 	// return name. Guard against NullPointerException by returning an
 	// empty string instead of null
-	return (*env)->NewStringUTF(env, (name != NULL ? name : "\0"));
+	return bindings_java_newString(env, (name != NULL ? name : "\0"));
 }
 
 /**
@@ -296,11 +317,11 @@ Java_org_gnome_glib_GValue_g_1value_1init__Ljava_lang_String_2
 	jstring _str
 )
 {
-	gchar* str;
+	const gchar* str;
 	GValue* value;
 
 	// translate
-	str = (gchar*) (*env)->GetStringUTFChars(env, _str, NULL);
+	str = bindings_java_getString(env, _str);
 	if (str == NULL) {
 		return 0; /* OutOfMemoryError already thrown */
 	}
@@ -313,7 +334,7 @@ Java_org_gnome_glib_GValue_g_1value_1init__Ljava_lang_String_2
 	g_value_set_string(value, str);
 
 	// clean up
-	(*env)->ReleaseStringUTFChars(env, _str, str);
+	bindings_java_releaseString(str);
 
 	// return address
 	return (jlong) value;
@@ -449,20 +470,20 @@ Java_org_gnome_glib_GValue_g_1value_1init_1enum
 	jint _num
 )
 {
-	gchar* name;
+	const gchar* name;
 	GType type;
 	gint num;
 	GValue* value;
 	
 	// translate type;
-	name = (gchar*) (*env)->GetStringUTFChars(env, _name, NULL);
+	name = bindings_java_getString(env, _name);
 	if (name == NULL) {
 		return 0; /* OutOfMemoryError already thrown */
 	}
 
 	type = g_type_from_name(name);
 
-	(*env)->ReleaseStringUTFChars(env, _name, name);
+	bindings_java_releaseString(name);
 	
 	// translate obj
 	num = (gint) _num;
@@ -661,7 +682,7 @@ Java_org_gnome_glib_GValue_g_1value_1get_1string
 	str = g_value_get_string(value);
 
 	// and return	
-	return (*env)->NewStringUTF(env, str);
+	return bindings_java_newString(env, str);
 }
 
 /**
