@@ -1,13 +1,34 @@
 /*
- * Button.java
+ * java-gnome, a UI library for writing GTK and GNOME programs from Java!
  *
- * Copyright (c) 2006-2008 Operational Dynamics Consulting Pty Ltd, and Others
- * 
- * The code in this file, and the library it is a part of, are made available
- * to you by the authors under the terms of the "GNU General Public Licence,
- * version 2" plus the "Classpath Exception" (you may link to this code as a
- * library into other programs provided you don't make a derivation of it).
- * See the LICENCE file for the terms governing usage and redistribution.
+ * Copyright © 2006-2010 Operational Dynamics Consulting, Pty Ltd and Others
+ *
+ * The code in this file, and the program it is a part of, is made available
+ * to you by its authors as open source software: you can redistribute it
+ * and/or modify it under the terms of the GNU General Public License version
+ * 2 ("GPL") as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE. See the GPL for more details.
+ *
+ * You should have received a copy of the GPL along with this program. If not,
+ * see http://www.gnu.org/licenses/. The authors of this program may be
+ * contacted through http://java-gnome.sourceforge.net/.
+ *
+ * Linking this library statically or dynamically with other modules is making
+ * a combined work based on this library. Thus, the terms and conditions of
+ * the GPL cover the whole combination. As a special exception (the
+ * "Claspath Exception"), the copyright holders of this library give you
+ * permission to link this library with independent modules to produce an
+ * executable, regardless of the license terms of these independent modules,
+ * and to copy and distribute the resulting executable under terms of your
+ * choice, provided that you also meet, for each linked independent module,
+ * the terms and conditions of the license of that module. An independent
+ * module is a module which is not derived from or based on this library. If
+ * you modify this library, you may extend the Classpath Exception to your
+ * version of the library, but you are not obligated to do so. If you do not
+ * wish to do so, delete this exception statement from your version.
  */
 package org.gnome.gtk;
 
@@ -23,10 +44,10 @@ package org.gnome.gtk;
  * Since Button is a Bin it strictly only has one child. Internally, however,
  * it may have both an icon image and some text (which is the look commonly
  * seen in the action buttons in Dialog boxes). You can add such an image to a
- * Button by calling {@link #setImage(Image) setImage()}; this works
- * alongside and with {@link #setLabel(String) setLabel()}. The machinery
- * within Button will manage creating the necessary internal structure
- * (HBoxes, Alignments, etc).
+ * Button by calling {@link #setImage(Image) setImage()}; this works alongside
+ * and with {@link #setLabel(String) setLabel()}. The machinery within Button
+ * will manage creating the necessary internal structure (HBoxes, Alignments,
+ * etc).
  * 
  * @author Andrew Cowie
  * @author Vreixo Formoso
@@ -95,12 +116,11 @@ public class Button extends Bin
      * Set the text showing in the Button.
      * 
      * <p>
-     * If you created an empty Button without a Label using
-     * {@link #Button() Button()}, this will create a Label nested in an
-     * Alignment for you. That <i>won't</i> work if you create an empty
-     * Button then put a custom Widget in place with
-     * {@link Container#add(Widget) add()} instead of employing this method
-     * and/or {@link #setImage(Image) setImage()}).
+     * If you created an empty Button without a Label using {@link #Button()
+     * Button()}, this will create a Label nested in an Alignment for you.
+     * That <i>won't</i> work if you create an empty Button then put a custom
+     * Widget in place with {@link Container#add(Widget) add()} instead of
+     * employing this method and/or {@link #setImage(Image) setImage()}).
      * 
      * @since 4.0.0
      */
@@ -219,59 +239,160 @@ public class Button extends Bin
      * emitted, but only if the cursor is still in the Button when the mouse
      * button is released. You're probably used to this behaviour without
      * realizing it.</i>
-     */
-    public interface CLICKED extends GtkButton.CLICKED
-    {
-        public void onClicked(Button source);
-    }
-
-    /**
-     * Hook up a handler to receive "clicked" events on this Button. A typical
-     * example of how this is used is as follows:
+     * 
+     * <h2>All about signal handling</h2>
+     * 
+     * <h3>Use an anonymous inner class</h3>
+     * 
+     * <p>
+     * In general, the way that java-gnome is intended to be used is for you
+     * to create an anonymous inner class right where you call the
+     * <code>connect()</code> method to hook up the signal. A typical example
+     * is as follows:
      * 
      * <pre>
-     * Button b;
-     *              
-     * b.connect(new Button.CLICKED() {
+     * final Button b;
+     * 
+     * b.connect(new Button.Clicked() {
      *     public void onClicked(Button source) {
-     *         // do something!
+     *         doSomething();
      *     }
      * }
      * </pre>
      * 
-     * <p>
-     * You can of course create a subclass of Button.CLICKED and then use
-     * instances of it if you have highly complicated algorithms to implement.
+     * This is the form we recommend; it has the advantage that your handler
+     * code is close to the code that declares and configures the Widget.
+     * Also, if variables are declared <code>final</code> then you can access
+     * them from within the nested anonymous class, and that makes things easy
+     * indeed.
+     * 
+     * <h3>Using a concrete instance</h3>
      * 
      * <p>
-     * If you implement Button.CLICKED in the class you're currently working
-     * on, then you use a technique called "self-delegation" which can
-     * sometimes work well;
+     * If you have more complex code, perhaps that you need to reuse across
+     * several Widgets, then you can of course create a concrete class that
+     * implements Button.Clicked and then use instances of it if you have
+     * highly complicated algorithms to implement:
+     * 
+     * <pre>
+     * class ComplexHandler implements Button.Clicked
+     * {
+     *     private int field;
+     * 
+     *     ComplexHandler(int param) {
+     *         this.field = param;
+     *     }
+     * 
+     *     public void onClicked(Button source) {
+     *         doSomethingVeryComplicatedWith(field);
+     *     }
+     * }
+     * </pre>
+     * 
+     * and then when you construct the Button:
+     * 
+     * <pre>
+     * b = new Button();
+     * b.connect(new ComplexHandler(42));
+     * </pre>
+     * 
+     * You can just as easily declare one and reuse it if you want to apply
+     * the same logic to a series of Button; this is where the
+     * <code>source</code> parameter that every signal callback signature
+     * includes.
+     * 
+     * <pre>
+     * final ComplexHandler handler;
+     * 
+     * handler = new ComplexHandler(42);
+     * ...
+     * b1.connect(handler);
+     * b2.connect(handler);
+     * b3.connect(handler);
+     * ...
+     * </pre>
+     * 
+     * <h3>Self-delegation</h3>
+     * 
+     * <p>
+     * If you implement Button.Clicked in the class you're currently working
+     * on, then you use a technique called "self-delegation" which can be
+     * useful in certain circumstances:
      * 
      * <pre>
      * b.connect(this);
      * </pre>
      * 
+     * This has disadvantage that you can only have one
+     * <code>onClicked()</code> method in your class, so if you have lots of
+     * Buttons then you lose the benefit of being able to connect different
+     * handlers to different Buttons.
+     * 
+     * <p>
+     * We recommend you use anonymous inner classes in-place whenever
+     * possible. In our experience, having the handler logic next to the code
+     * that declares an instance dramatically improves readability.
+     * 
+     * @author Andrew Cowie
      * @since 4.0.0
      */
+    public interface Clicked extends GtkButton.ClickedSignal
+    {
+        public void onClicked(Button source);
+    }
+
+    /**
+     * Hook up a handler to receive <code>Button.Clicked</code> events on this
+     * Button. See {@link Button.Clicked} for a detailed discussion of how to
+     * connect signals.
+     * 
+     * @since 4.0.0
+     */
+    public void connect(Clicked handler) {
+        GtkButton.connect(this, handler, false);
+    }
+
+    /**
+     * @deprecated
+     */
+    public interface CLICKED extends GtkButton.ClickedSignal
+    {
+    }
+
+    /**
+     * @deprecated
+     */
     public void connect(CLICKED handler) {
-        GtkButton.connect(this, handler);
+        assert false : "use Button.Clicked instead";
+        GtkButton.connect(this, handler, false);
     }
 
     /*
-     * ACTIVATE: "Applications should never connect to this signal, but use
-     * the 'clicked' signal."
-     */
-    /*
-     * ENTERED, PRESSED, etc: "deprecated"
+     * Button.Activate: "Applications should never connect to this signal, but
+     * use the 'clicked' signal."
+     * 
+     * Button.Entered, Button.Pressed, etc: deprecated.
      */
 
     /**
-     * Cause a <code>CLICKED</code> signal to be emitted.
+     * Cause a <code>Button.Clicked</code> signal to be emitted.
      * 
      * @since 4.0.6
      */
     public void emitClicked() {
         GtkButton.clicked(this);
     }
+
+    /**
+     * Set whether clicking the Button will cause the Button to grab the
+     * focus. The default is <code>true</code>, the behaviour you're
+     * accustomed to with regular Buttons. Setting this to <code>false</code>
+     * is only used in unusual situations like toolboxes where you don't want
+     * to steal focus away from where the main action is taking place.
+     * 
+     * @since 4.0.8
+     */
+    public void setFocusOnClick(boolean setting) {
+        GtkButton.setFocusOnClick(this, setting);
+    }
 }
