@@ -1,13 +1,34 @@
 /*
- * Dialog.java
+ * java-gnome, a UI library for writing GTK and GNOME programs from Java!
  *
- * Copyright (c) 2007-2008 Operational Dynamics Consulting Pty Ltd, and Others
+ * Copyright © 2007-2010 Operational Dynamics Consulting, Pty Ltd and Others
  *
- * The code in this file, and the library it is a part of, are made available
- * to you by the authors under the terms of the "GNU General Public Licence,
- * version 2" plus the "Classpath Exception" (you may link to this code as a
- * library into other programs provided you don't make a derivation of it).
- * See the LICENCE file for the terms governing usage and redistribution.
+ * The code in this file, and the program it is a part of, is made available
+ * to you by its authors as open source software: you can redistribute it
+ * and/or modify it under the terms of the GNU General Public License version
+ * 2 ("GPL") as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE. See the GPL for more details.
+ *
+ * You should have received a copy of the GPL along with this program. If not,
+ * see http://www.gnu.org/licenses/. The authors of this program may be
+ * contacted through http://java-gnome.sourceforge.net/.
+ *
+ * Linking this library statically or dynamically with other modules is making
+ * a combined work based on this library. Thus, the terms and conditions of
+ * the GPL cover the whole combination. As a special exception (the
+ * "Claspath Exception"), the copyright holders of this library give you
+ * permission to link this library with independent modules to produce an
+ * executable, regardless of the license terms of these independent modules,
+ * and to copy and distribute the resulting executable under terms of your
+ * choice, provided that you also meet, for each linked independent module,
+ * the terms and conditions of the license of that module. An independent
+ * module is a module which is not derived from or based on this library. If
+ * you modify this library, you may extend the Classpath Exception to your
+ * version of the library, but you are not obligated to do so. If you do not
+ * wish to do so, delete this exception statement from your version.
  */
 package org.gnome.gtk;
 
@@ -76,6 +97,8 @@ public class Dialog extends Window
      * the rest of the Dialog setup manually yourself, including calling
      * {@link Window#setTransientFor(Window) setTransientFor()} and
      * {@link Window#setTitle(String) setTitle()}.
+     * 
+     * @since 4.0.5
      */
     public Dialog() {
         super(GtkDialog.createDialog());
@@ -95,9 +118,9 @@ public class Dialog extends Window
      * @param title
      *            Title of the new Dialog.
      * @param parent
-     *            Transient parent of the dialog, or <code>null</code>. If
-     *            a parent is provided, the window manager will keep the
-     *            Dialog on top of it. If the Dialog is modal, it is highly
+     *            Transient parent of the dialog, or <code>null</code>. If a
+     *            parent is provided, the window manager will keep the Dialog
+     *            on top of it. If the Dialog is modal, it is highly
      *            recommended that you provide a parent Window, otherwise the
      *            user could be trying to interact with the main Window while
      *            the Dialog is blocking it, but hidden under other Window. In
@@ -108,6 +131,7 @@ public class Dialog extends Window
      *            blocks interaction with the other parts of the application.
      *            Note that you can also get a similar behaviour using the
      *            {@link #run() run()} method.
+     * @since 4.0.5
      */
     public Dialog(String title, Window parent, boolean modal) {
         super(GtkDialog.createDialogWithButtons(title, parent, modal ? DialogFlags.MODAL
@@ -116,6 +140,8 @@ public class Dialog extends Window
 
     /**
      * Add a Widget to the <var>main area</var> of the Dialog.
+     * 
+     * @since 4.0.5
      */
     public void add(Widget widget) {
         final VBox box;
@@ -126,12 +152,13 @@ public class Dialog extends Window
 
     /**
      * Adds an action {@link Button} with the given text as its label to the
-     * end of the Dialog's <var>action area</var>. The given ResponseType
-     * will be returned back in the Dialog's {@link Dialog.Response} signal
-     * when the Button added as a result of this call is clicked.
+     * end of the Dialog's <var>action area</var>. The given ResponseType will
+     * be returned back in the Dialog's {@link Dialog.Response} signal when
+     * the Button added as a result of this call is clicked.
      * 
      * @return the added Button. This is a convenience allowing you to hook up
      *         further handlers to the Button if necessary.
+     * @since 4.0.5
      */
     public Button addButton(String text, ResponseType response) {
         return (Button) GtkDialog.addButton(this, text, response.getResponseId());
@@ -141,11 +168,43 @@ public class Dialog extends Window
      * Add a Button whose icon and label are taken from a given Stock. It is,
      * as ever, recommended to use a Stock Button for common actions. See
      * {@link #addButton(String, ResponseType) addButton()}.
+     * 
+     * @since 4.0.5
      */
     public Button addButton(Stock stock, ResponseType response) {
         return (Button) GtkDialog.addButton(this, stock.getStockId(), response.getResponseId());
     }
 
+    /**
+     * Add a Button Widget you've already constructed to the action area of
+     * this Dialog.
+     * 
+     * <p>
+     * It is, as ever, recommended to use a Stock Button for common actions.
+     * See {@link #addButton(String, ResponseType) addButton()}.
+     * 
+     * <p>
+     * <i>You can pass any Activatable as the <code>widget</code>
+     * parameter.</i>
+     * 
+     * @since 4.0.14
+     */
+    public Button addButton(Widget widget, ResponseType response) {
+        GtkDialog.addActionWidget(this, widget, response.getResponseId());
+
+        /*
+         * The other addButton() signature returns Button, though the native
+         * code here actually takes a GtkActivatable. So workaround in this
+         * overload by returning Button if possible.
+         */
+
+        if (widget instanceof Button) {
+            return (Button) widget;
+        } else {
+            return null;
+        }
+    }
+
     /**
      * Block the rest of the application by running in a recursive main loop
      * until a {@link Dialog.Response} signal arises on the Dialog as a result
@@ -176,6 +235,7 @@ public class Dialog extends Window
      * @return the emitted response constant. If asking a question, you should
      *         check this against the various constants in the ResponseType
      *         class. Don't forget to <code>hide()</code> afterwards.
+     * @since 4.0.5
      */
     public ResponseType run() {
         final int value;
@@ -258,4 +318,14 @@ public class Dialog extends Window
             handler.onResponse(source, response);
         }
     }
+
+    /**
+     * Cause a <code>Dialog.Response</code> signal with the specified
+     * ResponseType to be emitted by this Dialog.
+     * 
+     * @since 4.0.9
+     */
+    public void emitResponse(ResponseType response) {
+        GtkDialog.response(this, response.getResponseId());
+    }
 }
