@@ -1,13 +1,34 @@
 /*
- * TextBuffer.java
+ * java-gnome, a UI library for writing GTK and GNOME programs from Java!
  *
- * Copyright (c) 2007-2009 Operational Dynamics Consulting Pty Ltd, and Others
+ * Copyright © 2007-2010 Operational Dynamics Consulting, Pty Ltd and Others
  *
- * The code in this file, and the library it is a part of, are made available
- * to you by the authors under the terms of the "GNU General Public Licence,
- * version 2" plus the "Classpath Exception" (you may link to this code as a
- * library into other programs provided you don't make a derivation of it).
- * See the LICENCE file for the terms governing usage and redistribution.
+ * The code in this file, and the program it is a part of, is made available
+ * to you by its authors as open source software: you can redistribute it
+ * and/or modify it under the terms of the GNU General Public License version
+ * 2 ("GPL") as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE. See the GPL for more details.
+ *
+ * You should have received a copy of the GPL along with this program. If not,
+ * see http://www.gnu.org/licenses/. The authors of this program may be
+ * contacted through http://java-gnome.sourceforge.net/.
+ *
+ * Linking this library statically or dynamically with other modules is making
+ * a combined work based on this library. Thus, the terms and conditions of
+ * the GPL cover the whole combination. As a special exception (the
+ * "Claspath Exception"), the copyright holders of this library give you
+ * permission to link this library with independent modules to produce an
+ * executable, regardless of the license terms of these independent modules,
+ * and to copy and distribute the resulting executable under terms of your
+ * choice, provided that you also meet, for each linked independent module,
+ * the terms and conditions of the license of that module. An independent
+ * module is a module which is not derived from or based on this library. If
+ * you modify this library, you may extend the Classpath Exception to your
+ * version of the library, but you are not obligated to do so. If you do not
+ * wish to do so, delete this exception statement from your version.
  */
 package org.gnome.gtk;
 
@@ -16,8 +37,6 @@ import java.util.Collection;
 import org.gnome.gdk.Pixbuf;
 import org.gnome.glib.Object;
 
-import static org.gnome.gtk.TextTagTable.getDefaultTable;
-
 /**
  * A TextBuffer is a powerful mechanism for storing and manipulating text. It
  * exists primarily to be the model that backs the view of one or more
@@ -148,11 +167,32 @@ public class TextBuffer extends Object
      */
     public static final char OBJECT_REPLACEMENT_CHARACTER = 0xFFFC;
 
+    private static TextTagTable defaultTable;
+
+    /**
+     * We maintain a single default TextTagTable to facilitate no-arg
+     * convenience constructors for TextTag and TextBuffer. Get (and
+     * initialize if necessary) this table.
+     */
+    /*
+     * synchronized?
+     */
+    protected static TextTagTable getDefaultTable() {
+        if (defaultTable == null) {
+            defaultTable = new TextTagTable();
+        }
+        return defaultTable;
+    }
+
     private final boolean usingDefaultTable;
 
     protected TextBuffer(long pointer) {
+        this(pointer, false);
+    }
+
+    protected TextBuffer(long pointer, boolean usingDefaultTable) {
         super(pointer);
-        usingDefaultTable = false;
+        this.usingDefaultTable = usingDefaultTable;
     }
 
     /**
@@ -169,8 +209,7 @@ public class TextBuffer extends Object
      * @since 4.0.9
      */
     public TextBuffer() {
-        super(GtkTextBuffer.createTextBuffer(getDefaultTable()));
-        usingDefaultTable = true;
+        this(GtkTextBuffer.createTextBuffer(getDefaultTable()), true);
     }
 
     /**
@@ -180,8 +219,7 @@ public class TextBuffer extends Object
      * @since 4.0.9
      */
     public TextBuffer(TextTagTable tags) {
-        super(GtkTextBuffer.createTextBuffer(tags));
-        usingDefaultTable = false;
+        this(GtkTextBuffer.createTextBuffer(tags), false);
     }
 
     /**
@@ -414,7 +452,13 @@ public class TextBuffer extends Object
      * Like {@link #insert(TextIter, String) insert()}, but the insertion will
      * not occur if <code>pos</code> points to a non-editable location in the
      * buffer - meaning that it is enclosed in TextTags that mark the area
-     * non-editable.<br/>
+     * non-editable.
+     * 
+     * <p>
+     * This method is conducted as a user action, so the
+     * <code>TextBuffer.BeginUserAction</code> and
+     * <code>TextBuffer.EndUserAction</code> signals will be raised before and
+     * after carrying out the insertion, respectively.
      * 
      * @param pos
      *            Position to insert at.
@@ -433,6 +477,27 @@ public class TextBuffer extends Object
         GtkTextBuffer.insertInteractive(this, pos, text, -1, defaultEditability);
     }
 
+    /**
+     * Delete text like {@link #delete(TextIter, TextIter) delete()}, but only
+     * if the range given is editable.
+     * 
+     * <p>
+     * See {@link #insertInteractive(TextIter, String, boolean)
+     * insertInteractive()} for a discussion of the
+     * <code>defaultEditability</code> parameter.
+     * 
+     * <p>
+     * This method is conducted as a user action, so the
+     * <code>TextBuffer.BeginUserAction</code> and
+     * <code>TextBuffer.EndUserAction</code> signals will be raised before and
+     * after carrying out the deletion.
+     * 
+     * @since 4.0.13
+     */
+    public void deleteInteractive(TextIter start, TextIter end, boolean defaultEditability) {
+        GtkTextBuffer.deleteInteractive(this, start, end, defaultEditability);
+    }
+
     /**
      * Inserts an image at the cursor position.
      * 
@@ -1126,4 +1191,29 @@ public class TextBuffer extends Object
     public void connect(EndUserAction handler) {
         GtkTextBuffer.connect(this, handler, false);
     }
+
+    /**
+     * Tell a currently active <code>TextBuffer.InsertText</code> signal
+     * emission to stop.
+     * 
+     * <p>
+     * Calling this only makes sense within the scope of a normal handler of
+     * that signal; the effect is to prevent insertion by GTK's default
+     * handler.
+     * 
+     * @since 4.0.13
+     */
+    public void stopInsertText() {
+        GtkTextBufferOverride.stopInsertText(this);
+    }
+
+    /**
+     * Tell a currently active <code>TextBuffer.DeleteRange</code> signal
+     * emission to stop.
+     * 
+     * @since 4.0.13
+     */
+    public void stopDeleteRange() {
+        GtkTextBufferOverride.stopDeleteRange(this);
+    }
 }
