@@ -1,13 +1,34 @@
 /*
- * TreeViewColumn.java
+ * java-gnome, a UI library for writing GTK and GNOME programs from Java!
  *
- * Copyright (c) 2007-2008 Operational Dynamics Consulting Pty Ltd
+ * Copyright © 2007-2010 Operational Dynamics Consulting, Pty Ltd and Others
  *
- * The code in this file, and the library it is a part of, are made available
- * to you by the authors under the terms of the "GNU General Public Licence,
- * version 2" plus the "Classpath Exception" (you may link to this code as a
- * library into other programs provided you don't make a derivation of it).
- * See the LICENCE file for the terms governing usage and redistribution.
+ * The code in this file, and the program it is a part of, is made available
+ * to you by its authors as open source software: you can redistribute it
+ * and/or modify it under the terms of the GNU General Public License version
+ * 2 ("GPL") as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE. See the GPL for more details.
+ *
+ * You should have received a copy of the GPL along with this program. If not,
+ * see http://www.gnu.org/licenses/. The authors of this program may be
+ * contacted through http://java-gnome.sourceforge.net/.
+ *
+ * Linking this library statically or dynamically with other modules is making
+ * a combined work based on this library. Thus, the terms and conditions of
+ * the GPL cover the whole combination. As a special exception (the
+ * "Claspath Exception"), the copyright holders of this library give you
+ * permission to link this library with independent modules to produce an
+ * executable, regardless of the license terms of these independent modules,
+ * and to copy and distribute the resulting executable under terms of your
+ * choice, provided that you also meet, for each linked independent module,
+ * the terms and conditions of the license of that module. An independent
+ * module is a module which is not derived from or based on this library. If
+ * you modify this library, you may extend the Classpath Exception to your
+ * version of the library, but you are not obligated to do so. If you do not
+ * wish to do so, delete this exception statement from your version.
  */
 package org.gnome.gtk;
 
@@ -23,6 +44,7 @@ package org.gnome.gtk;
  * {@link TreeView#appendColumn() appendColumn()}.
  * 
  * @author Andrew Cowie
+ * @author Stefan Prelle
  * @since 4.0.5
  */
 /*
@@ -47,25 +69,38 @@ public class TreeViewColumn extends Object implements CellLayout
 
     /**
      * Set the title to be used for this TreeViewColumn in the TreeView's
-     * header row. In addition to being descriptive, the title are what you
+     * header row. In addition to being descriptive, this title is what you
      * click to cause this column to be the one that is sorting the TreeView
-     * (assuming you've enabled the columns to be sortable with TreeView's
-     * {@link TreeView#setHeadersClickable(boolean) setHeadersClickable(true)}
-     * and FIXME, and that the titles are visible in the first place, ie the
-     * header row hasn't been turned off via TreeView's
-     * {@link TreeView#setHeadersVisible(boolean) setHeadersVisible(false)}).
+     * [assuming you've enabled the columns to be sortable by calling the
+     * enclosing TreeView's {@link TreeView#setHeadersClickable(boolean)
+     * setHeadersClickable(true)} and this TreeViewColumn's
+     * {@link TreeViewColumn#setSortColumn(DataColumn) setSortColumn()}, and
+     * that the titles are visible in the first place, ie the header row
+     * hasn't been turned off via TreeView's
+     * {@link TreeView#setHeadersVisible(boolean) setHeadersVisible(false)}].
      */
     public void setTitle(String title) {
         GtkTreeViewColumn.setTitle(this, title);
     }
 
+    /**
+     * Gets the title of this TreeViewColumn. Please see
+     * {@link #setTitle(String) setTitle()} for further information about
+     * TreeViewColumn titles.
+     * 
+     * @since 4.0.8
+     */
+    public String getTitle() {
+        return GtkTreeViewColumn.getTitle(this);
+    }
+
     /**
      * Set whether the column header allows itself to be clicked. The default
      * is <code>false</code>, not to be clickable, also meaning that header
      * won't take the keyboard focus either. Since calling
      * {@link #setSortColumn(DataColumn) setSortColumn()} will set
-     * <var>clickable</var> to <code>true</code>, so you only need this
-     * for the case where you assign a sort order to a column, but don't want
+     * <var>clickable</var> to <code>true</code>, so you only need this for
+     * the case where you assign a sort order to a column, but don't want
      * users to be able to change the sorting column or its order.
      */
     public void setClickable(boolean setting) {
@@ -77,21 +112,22 @@ public class TreeViewColumn extends Object implements CellLayout
      * sorting be enabled.
      * 
      * <p>
-     * Calling this also makes the TreeViewColumn <var>clickable</var>, but
-     * it does not automatically engage the sorting (maybe you only want
-     * sorting to be optional and only active if the user clicks a header,
-     * certainly if you've set sort information for many columns you will have
-     * to indicate explicitly which one is to be active first, etc). Use
-     * {@link #clicked()} to activate this column as the one doing the
-     * sorting.
+     * Calling this also makes the TreeViewColumn <var>clickable</var>, but it
+     * does not automatically engage the sorting (maybe you only want sorting
+     * to be optional and only active if the user clicks a header, certainly
+     * if you've set sort information for many columns you will have to
+     * indicate explicitly which one is to be active first, etc). Use
+     * {@link #emitClicked() emitClicked()} to activate this column as the one
+     * doing the sorting.
      * 
      * <p>
-     * <b>Warning:</b><br/>This only works if the TreeModel being presented
-     * by this TreeView is <var>sortable</var>, that is, if that model
-     * implements TreeSortable. Since almost everything in GTK does so, you
-     * ordinarily don't need to worry about this. The one that doesn't is
-     * TreeModelFilter, so if you're using a filtered model make sure you wrap
-     * it in a TreeModelSort before adding it to the TreeView.
+     * <b>Warning:</b><br/>
+     * This only works if the TreeModel being presented by this TreeView is
+     * <var>sortable</var>, that is, if that model implements TreeSortable.
+     * Since almost everything in GTK does so, you ordinarily don't need to
+     * worry about this. The one that doesn't is TreeModelFilter, so if you're
+     * using a filtered model make sure you wrap it in a TreeModelSort before
+     * adding it to the TreeView.
      */
     public void setSortColumn(DataColumn column) {
         GtkTreeViewColumn.setSortColumnId(this, column.getOrdinal());
@@ -99,15 +135,23 @@ public class TreeViewColumn extends Object implements CellLayout
 
     /**
      * Activate this column's sorting by sending a click the TreeViewColumn
-     * header. Causes <code>CLICKED</code> to fire, but more importantly
-     * this causes this vertical to become the active column dictating the
-     * ordering of the TreeView as a whole (assuming the vertical is
-     * {@link #setClickable(boolean) setClickable(true)} and assuming a
-     * DataColumn has been set with
+     * header. Causes <code>TreeViewColumn.Clicked</code> to fire, but more
+     * importantly this causes this vertical to become the active column
+     * dictating the ordering of the TreeView as a whole (assuming the
+     * vertical is {@link #setClickable(boolean) setClickable(true)} and
+     * assuming a DataColumn has been set with
      * {@link #setSortColumn(DataColumn) setSortColumn()} to indicate the
      * ordering).
+     * 
+     * @since 4.0.8
      */
+    public void emitClicked() {
+        GtkTreeViewColumn.clicked(this);
+    }
+
+    /** @deprecated */
     public void clicked() {
+        assert false : "use emitClicked() instead";
         GtkTreeViewColumn.clicked(this);
     }
 
@@ -124,9 +168,8 @@ public class TreeViewColumn extends Object implements CellLayout
      * The default for new TreeViewColumns is <code>false</code>. The last
      * (right-most) column is treated specially, however. Extra space given to
      * the TreeView as a whole will automatically go to the last vertical
-     * column (ie, ignoring this being set <code>false</code>) unless one
-     * or more other columns have had <var>expand</var> set to
-     * <code>true</code>.
+     * column (ie, ignoring this being set <code>false</code>) unless one or
+     * more other columns have had <var>expand</var> set to <code>true</code>.
      */
     public void setExpand(boolean setting) {
         GtkTreeViewColumn.setExpand(this, setting);
@@ -174,7 +217,9 @@ public class TreeViewColumn extends Object implements CellLayout
      * Permit the user to change the width of the columns in a TreeView by
      * clicking on the boundary of a column header and dragging it. Setting
      * this <code>true</code> will implicitly set
-     * {@link TreeViewColumnSizing#GROW_ONLY GROW_ONLY}.
+     * {@link TreeViewColumnSizing#GROW_ONLY GROW_ONLY}. On the other hand
+     * setting {@link TreeViewColumnSizing#AUTOSIZE AUTOSIZE} will implicitly
+     * disable resizing.
      * 
      * @since 4.0.6
      */
@@ -186,8 +231,8 @@ public class TreeViewColumn extends Object implements CellLayout
      * Set the mode constant that will be used to determine the width
      * behaviour of TreeViewColumns when data is changed in the model causing
      * a CellRenderer to need to redraw a cell. Note that this works in
-     * conjunction with other sizing methods ({@link #setResizable(boolean) setResizable()},
-     * {@link #setMinWidth(int) setMinWidth()},
+     * conjunction with other sizing methods ({@link #setResizable(boolean)
+     * setResizable()}, {@link #setMinWidth(int) setMinWidth()},
      * {@link #setFixedWidth(int) setFixedWidth()}, etc and there are quite a
      * number of permutations available as a result.
      * 
@@ -196,13 +241,55 @@ public class TreeViewColumn extends Object implements CellLayout
      * 
      * @since 4.0.6
      */
-    /*
-     * TODO I've got a feeling that the different permutations are not all
-     * equal and that some settings will override others. It would be nice if
-     * whoever exposes setMinWidth(), setFixedWidth(), etc would research this
-     * and add some documentation as to their interactions.
-     */
     public void setSizing(TreeViewColumnSizing type) {
         GtkTreeViewColumn.setSizing(this, type);
     }
+
+    /**
+     * Sets the minimal width this column can shrink to. Works in all
+     * {@link TreeViewColumnSizing} variants.
+     * 
+     * @param width
+     *            Column width in pixel
+     * 
+     * @since 4.0.8
+     */
+    public void setMinWidth(int width) {
+        GtkTreeViewColumn.setMinWidth(this, width);
+    }
+
+    /**
+     * If (and only if) operating with the {@link TreeViewColumnSizing#FIXED
+     * FIXED} TreeViewSizing mode, this defines the fixed column width in
+     * pixels. If {@link #setMinWidth(int)} setMinWidth} or
+     * {@link #setMaxWidth(int)} have been called and the fixed width is
+     * outside these bounds Min- or MaxWidth take precedence.
+     * 
+     * <p>
+     * The fixed width can be overridden by the user if
+     * {@link #setResizable(boolean) setResizable()} has been set to
+     * <code>true</code>.
+     * 
+     * @param width
+     *            Column width in pixels
+     * 
+     * @since 4.0.8
+     */
+    public void setFixedWidth(int width) {
+        GtkTreeViewColumn.setFixedWidth(this, width);
+    }
+
+    /**
+     * Sets the maximum width this column can grow. Works in all
+     * {@link TreeViewColumnSizing} variants. If not set, the column growth is
+     * not limited.
+     * 
+     * @param width
+     *            Column width in pixels
+     * 
+     * @since 4.0.8
+     */
+    public void setMaxWidth(int width) {
+        GtkTreeViewColumn.setMaxWidth(this, width);
+    }
 }
