@@ -1,13 +1,34 @@
 /*
- * bindings_java_util.c
+ * java-gnome, a UI library for writing GTK and GNOME programs from Java!
  *
- * Copyright (c) 2006-2009 Operational Dynamics Consulting Pty Ltd and Others
- * 
- * The code in this file, and the library it is a part of, are made available
- * to you by the authors under the terms of the "GNU General Public Licence,
- * version 2" plus the "Classpath Exception" (you may link to this code as a
- * library into other programs provided you don't make a derivation of it).
- * See the LICENCE file for the terms governing usage and redistribution.
+ * Copyright © 2006-2010 Operational Dynamics Consulting, Pty Ltd and Others
+ *
+ * The code in this file, and the program it is a part of, is made available
+ * to you by its authors as open source software: you can redistribute it
+ * and/or modify it under the terms of the GNU General Public License version
+ * 2 ("GPL") as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE. See the GPL for more details.
+ *
+ * You should have received a copy of the GPL along with this program. If not,
+ * see http://www.gnu.org/licenses/. The authors of this program may be
+ * contacted through http://java-gnome.sourceforge.net/.
+ *
+ * Linking this library statically or dynamically with other modules is making
+ * a combined work based on this library. Thus, the terms and conditions of
+ * the GPL cover the whole combination. As a special exception (the
+ * "Claspath Exception"), the copyright holders of this library give you
+ * permission to link this library with independent modules to produce an
+ * executable, regardless of the license terms of these independent modules,
+ * and to copy and distribute the resulting executable under terms of your
+ * choice, provided that you also meet, for each linked independent module,
+ * the terms and conditions of the license of that module. An independent
+ * module is a module which is not derived from or based on this library. If
+ * you modify this library, you may extend the Classpath Exception to your
+ * version of the library, but you are not obligated to do so. If you do not
+ * wish to do so, delete this exception statement from your version.
  */
 
 #include <jni.h>
@@ -183,7 +204,7 @@ bindings_java_throw
 }
 
 void 
-bindings_java_throw_gerror
+bindings_java_throwGlibException
 (
 	JNIEnv* env, 
 	GError* error
@@ -283,8 +304,8 @@ bindings_java_debug
 {
 	jclass cls;
 	jmethodID mid = NULL;
-	jobject str;
-	gchar* name;
+	jobject _name;
+	const gchar* name;
 
 	cls = (*env)->FindClass(env, "java/lang/Object");
 	if ((*env)->ExceptionCheck(env)) {
@@ -298,8 +319,8 @@ bindings_java_debug
 		g_error("No methodID?");
 	}
 
-	str = (*env)->CallObjectMethod(env, obj, mid);
-	if (str == NULL) {
+	_name = (*env)->CallObjectMethod(env, obj, mid);
+	if (_name == NULL) {
 		(*env)->ExceptionDescribe(env);
 		g_error("null?");
 	}
@@ -308,7 +329,7 @@ bindings_java_debug
 		g_error("No String");
 	}
 
-	name = (gchar*) (*env)->GetStringUTFChars(env, str, NULL);
+	name = bindings_java_getString(env, _name);
 	if (name == NULL) {
 		(*env)->ExceptionDescribe(env);
 		g_error("OOM?");
@@ -319,6 +340,7 @@ bindings_java_debug
 	}
 
 	g_debug("obj.toString(): %s", name);
+	bindings_java_releaseString(name);
 }
 
 
@@ -386,3 +408,125 @@ bindings_java_logging_init
 {
 	g_log_set_default_handler(bindings_java_logging_handler, NULL);
 }
+
+/*
+ * Convert a Java String object into a NULL-terminated UTF-8 string
+ *
+ * This serves as a replacement for JNI's GetStringUTFChars(). It uses GLib's
+ * UTF-16 to UTF-8 conversion function, which theoretically is a compliant
+ * implementation, unlike Java's.
+ *
+ * You must call bindings_java_releaseString() on the returned gchar*!
+ */
+const gchar*
+bindings_java_getString
+(
+	JNIEnv* env,
+	jstring _str
+)
+{
+	const jchar* _pointer;
+	const gunichar2* pointer;
+	gchar* str;
+	jsize _len;
+	glong len;
+	GError* error = NULL;
+
+	if (_str == NULL) {
+		return NULL;
+	}
+
+	/*
+	 * This is strange; it's tempting to assume that we can wait for the
+	 * NULL termination, but testing showed that there was sometimes junk
+	 * characters at the end. It seems safest just to ask the VM how long
+	 * the char[] is.
+	 */
+
+	_len = (*env)->GetStringLength(env, _str);
+	len = (glong) _len;
+
+	/*
+	 * Get a pointer to the underlying character array. With any luck, the
+	 * data will not be copied.
+	 */
+	// WARNING uses the JVM critical code path
+	_pointer = (*env)->GetStringCritical(env, _str, NULL);
+	if (G_UNLIKELY(_pointer == NULL)) {
+		return NULL; // Java Exception already thrown
+	}
+	pointer = (const gunichar2*) _pointer;
+
+
+	// call conversion function
+	str = g_utf16_to_utf8(pointer, len, NULL, NULL, &error);
+
+	(*env)->ReleaseStringCritical(env, _str, _pointer);
+	// WARNING end critical section
+
+	// cleanup
+	if (G_UNLIKELY(error != NULL)) {
+		bindings_java_throwGlibException(env, error);
+		return NULL;
+	}
+
+	return str;
+}
+
+/*
+ * Convert a NULL-terminated UTF-8 string into a Java String object.
+ *
+ * This serves as a replacement for JNI's NewStringUTF(), using GLib's
+ * theoretically correct UTF-8 to UTF-16 conversion function, rather than
+ * Java's internal non-compliant encoding.
+ */
+jstring
+bindings_java_newString
+(
+	JNIEnv* env,
+	const gchar* str
+)
+{
+	gunichar2* pointer;
+	const jchar* _pointer;
+	jstring _result;
+	glong num;
+	GError* error = NULL;
+
+	if (str == NULL) {
+		return NULL;
+	}
+
+	// call conversion function
+	pointer = g_utf8_to_utf16(str, -1, NULL, &num, &error);
+
+	if (G_UNLIKELY(error != NULL)) {
+		bindings_java_throwGlibException(env, error);
+		return NULL;
+	}
+
+	// construct object
+	_pointer = (const jchar*) pointer;
+	_result = (*env)->NewString(env, _pointer, num);
+
+	// cleanup
+	g_free(pointer);
+
+	return _result;
+}
+
+/*
+ * Free a String as created by bindings_java_getString().
+ *
+ * This is largely just an exercise in getting us around the const qualifier in
+ * const-gchar* types, but it has the benefit of keeping our String conversion
+ * utilities symmetric with the JNI API.
+ */
+void
+bindings_java_releaseString
+(
+	const gchar* str
+)
+{
+	g_free((gpointer) str);
+}
